import com.visibleworkings.trace.TraceLogList;
import com.visibleworkings.trace.TraceParsedMessage;
import java.util.List;
import java.util.Iterator;
import java.util.Stack;
import java.io.File;
import java.io.IOException;

/**
 * Base class for programs that read and check trace logs.
 */

public class LogChecker {

    /* ************************************************************
     *  Maintain a stack of context messages that orient reader when
     *  something fails. 
     *************************************************************/
    static Stack context = new Stack();

    protected static void pushContext(String text) {
        context.push(text);
    }

    protected static void popContext() {
        context.pop();
    }


    /* ************************************************************
     * Override these methods if you want to change the way failure or
     * success is reported.
     *************************************************************/

    static protected void failureHeader(String comment) { 
            System.out.println("###################################################");
            System.out.println("#################### FAILURE ######################");

            if (!context.empty()) {
                System.out.println("Found a problem in the following context:");
                Iterator it = context.iterator();
                while (it.hasNext()) {
                    System.out.println(it.next().toString());
                }
            }
            if (comment!=null) {
                System.out.println("So what's the problem?");
                System.out.println(comment);
            }
    }

    static protected void failureTrailer() {
    }
    
    static protected void success() { // silence is golden
    }



    /* ************************************************************
     * Increasingly elaborate assertions.
     *************************************************************/

    static void fail(String comment) {
        failureHeader(comment);
        failureTrailer();
        System.exit(1);
    }

    static void assert(boolean shouldBeTrue, String comment) {
        if (!shouldBeTrue) {
            fail(comment);
        }
    }

    static void assertEquals(Object expected, Object gotten,
                             String comment) {
        if (!expected.equals(gotten)) {
            failureHeader(comment);
            System.out.println("Expected: " + expected);
            System.out.println("Got: " + gotten);
            failureTrailer();
            System.exit(1);
        }
    }

    // Allow an alternate value to compare to the Gotten object.
    static void assertEquals(Object expected, Object gotten,
                             Object alternate,
                             String comment) {
        if (!gotten.equals(expected) && !gotten.equals(alternate)) {
            failureHeader(comment);
            System.out.println("Expected: '" + expected + "' or '" + alternate + "'");
            System.out.println("Got: '" + gotten + "'");
            failureTrailer();
            System.exit(1);
        }
    }

    // Compare integer fields for parseability and in-rangedness.
    static void assertRange(String supposedInt, String kind,
                            int min, int max) { 
        int intVal = -1960;
        try {
            intVal = Integer.parseInt(supposedInt);
        } catch (NumberFormatException e) {
            fail(supposedInt + ", a " + kind + ", is not in the right format.");
        }
        
        assert(intVal >= min && intVal <= max,
            supposedInt + ", a " + kind + ", is not in the range [" +
            min + ", " + max + "].");
    }

    /* ************************************************************
     * Assertions about the contents of a log message.
     *************************************************************/

    /**
     * Check all values of a parsed log message. Some fields can be
     * null, meaning that they're not checked. (Exception: the
     * subsystem must always be given.)
     * The date is impossible to check exactly, so the only check
     * is that it's a valid date in the recent past.
     * Location fields (line number, method name, filename) are
     * allowed to have their "?" values (e.g., "line?").
     */
    static void assertValues(TraceParsedMessage msg,
                             String method, String file, String line,
                             String level, String subsystem, String message) {
        pushContext("... looking at this trace message: " +
                     System.getProperty("line.separator") +
                     msg.original);
        assertDateInContext(msg);
        assertStringsInContext(msg, method, file, line, level,
                               subsystem, message); 
        popContext();
    }

    static private void assertDateInContext(TraceParsedMessage msg) {
        assertRange(msg.year, "year", 1960, 3000);
        assertRange(msg.month, "month", 1, 12);
        assertRange(msg.day, "day", 1, 31);
        assertRange(msg.hour, "hour", 0, 23);
        assertRange(msg.minute, "minute", 0, 59);
        assertRange(msg.second, "second", 0, 59);
        assertRange(msg.milli, "millisecond", 0, 999);

        assert(msg.isRecent(), "The date looks too old or is in the future. Make sure you just created the logs.");
    }

    static private void assertStringsInContext(TraceParsedMessage msg,
                                               String method, String file,
                                               String line, 
                                               String level, String subsystem,
                                               String message) {
        if (method != null) {
            assertEquals(method, msg.method, "method?", "method is wrong");
        }
        if (file != null) {
            assertEquals(file, msg.file, "file?", "file is wrong");
        }
        if (line != null) {
            assertEquals(line, msg.line, "line?", "line is wrong");
        }
        if (level != null) {
            assertEquals(level, msg.level, "trace level is wrong");
        }
        assertEquals(subsystem, msg.subsystem, "subsystem is wrong");
        if (message != null) { 
            assertEquals(message, msg.message, "trace message is wrong");
        }
    }


    /* ************************************************************
     * Assertions about log files.
     *************************************************************/

    static protected void assertNumLogs(TraceLogList list, int expected) {
        int actual = list.size();
        assert(actual == expected,
               "The number of matching files is " + actual
               + ". It should be " + expected + ".");
    }

    static protected void assertLogName(TraceLogList list, int index,
                                        String name) {
        assertEquals(name,
                     ((File)list.get(index)).getName(),
                    "The log name is incorrect.");
    }
        
    static protected void assertLastLogName(TraceLogList list, String name) {
        assertLogName(list, list.size()-1, name);
    }

    static protected void assertNumMessages(List list, int expected) {
        int actual = list.size();
        assert(actual == expected,
               "The number of messages is " + actual
               + ". It should be " + expected + ".");
    }

    /* ************************************************************
     *  Fetching things to check from files.
     *************************************************************/

    protected static TraceLogList openLogs(File dir, String tag) {
        TraceLogList list = null;
        try {
            list = new TraceLogList(dir, tag);
        } catch (IOException e) {
            fail("Couldn't open log files: " + e.getMessage());
        }
        return list;
    }

    protected static List logMessages(TraceLogList list) { 
        List msgs = null;
        try {
            msgs = list.convertToMessages();
        } catch (IOException e) {
            fail("Couldn't read messages: " + e.getMessage());
        }
        return msgs;
    }
}
