package com.visibleworkings.trace;

/*
The contents of this file are subject to the Electric Communities E Open 
Source Code License Version 1.0 (the "License"); you may not use this file 
except in compliance with the License. You may obtain a copy of the License 
at http://www.communities.com/EL/.

Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for 
the specific language governing rights and limitations under the License.

The Original Code is the Distributed E Language Implementation, released 
July 20, 1998. 

The Initial Developer of the Original Code is Electric Communities.  
Copyright (C) 1998 Electric Communities. All Rights Reserved. 

Contributor(s): ______________________________________.
*/
/*
 *  Original version February 2001 by Brian Marick.
 */

import java.util.LinkedList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Comparator;
import java.util.List;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
// we work all day in the import factory...
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader; 

/**
 * TraceLogList is a List of File objects that correspond to trace log
 * files with the same TraceLog_name or TraceLog_tag. 
 */

final public class TraceLogList extends LinkedList {
    static public final boolean ON = true;

    /**
     * Discover a list of trace logs in the given directory that match
     * the given tagOrName. The list is sorted so that logs with more
     * recent messages come later in the list.
     *
     * Sorting currently only works for filenames of the form
     * <tag>[.<timestamp>][.<backup>].txt. Filenames of the form
     * <name>[.<backup>] (using Tracelog_name) are a kind of dumb idea
     * anyway - gratuitous flexibility.
     */
    public TraceLogList(File directory, String tagOrName)
        throws IOException {

        super();

        class Filter implements FilenameFilter {
            String myPrefix;

            Filter(String tagOrName) {
                myPrefix = tagOrName + ".";
            }

            public boolean accept(File dir, String name) {
                return name.startsWith(myPrefix);
            }
        }

        File[] files = directory.listFiles(new Filter(tagOrName));
        if (files == null) {
            throw new IOException(directory + " is unreadable.");
        }

        if (files.length == 0) {
            throw new IOException("No files match " + tagOrName);
        }
        
        class FileComparator implements Comparator {
            public int compare(Object o1, Object o2) {
                String name1 = ((File)o1).getName();
                String name2 = ((File)o2).getName();
                return name1.compareTo(name2);
            }
        }

        Arrays.sort(files, new FileComparator());
        addAll(Arrays.asList(files));
    }

    /*
     * Convert this list to TraceParsedMessages by parsing the
     * files. Note that the messages are concatenated into one big
     * list; file boundaries are not preserved. Any kind of error
     * throws an IOException. 
     */
    public List convertToMessages() throws IOException {
        Iterator i = iterator();
        LinkedList result = new LinkedList();

        while (i.hasNext()) {
            addContentsOfOneFile((File)i.next(), result);
        }
        return result;
    } 

    private void addContentsOfOneFile(File f, LinkedList list) 
        throws IOException {
        
        BufferedReader reader = 
            new BufferedReader(
                new InputStreamReader(
                    new FileInputStream(f)));

        while (true) { 
            String line1 = reader.readLine();
            if (line1 == null) {
                return;
            }
            
            String line2 = reader.readLine();
            if (line2 == null) { 
                throw new IOException("Partial message " + line1);
            }

            list.addLast(new TraceParsedMessage(line1, line2));
        }
    }

    public String toString() {
        Iterator i = iterator();
        String nl = System.getProperty("line.separator");
        String retval = "TraceLogList {" + nl;

        while (i.hasNext()) {
            File f = (File) i.next();
            retval += f + nl;
        }

        return retval + "}";
    }
            
}
