package com.visibleworkings.trace;

/*
The contents of this file are subject to the Electric Communities E Open 
Source Code License Version 1.0 (the "License"); you may not use this file 
except in compliance with the License. You may obtain a copy of the License 
at http://www.communities.com/EL/.

Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for 
the specific language governing rights and limitations under the License.

The Original Code is the Distributed E Language Implementation, released 
July 20, 1998. 

The Initial Developer of the Original Code is Electric Communities.  
Copyright (C) 1998 Electric Communities. All Rights Reserved. 

Contributor(s): ______________________________________.
*/
/*
 *  Trace and Logging Package.  Written by Brian Marick,
 *  July-September 1997, for Electric Communities, Inc.
 *  Copyright (c) 1997 by Electric Communities.  All Rights Reserved.
 */

import java.io.StringWriter;
import java.io.PrintWriter;


/**
 * This class is used to convert a trace message into a string.  It
 * is used (as opposed to toString()) when certain of the fields need
 * to be omitted.
 * <p>
 * Future:  IFC might allow the objects contained in trace messages to
 * be displayed as special glyphs, even clickable buttons.
 * <p>
 * By default, everything is shown.
 */
class TraceMessageStringifier {
    // See the setter methods for the meanings of these booleans.
    private boolean myShowDate = true;
    private boolean myShowTime = true;
    private boolean myShowLocation = true;
    private boolean myShowSubsystem = true;
    private boolean myShowLevel = true;
    private String lineSeparator = System.getProperty("line.separator");

    // A minor concession to efficiency, as this is critical path.
    // Used in toString.
    private StringBuffer buffer = new StringBuffer(200);

    /**
     * Should the message include the date at which the
     * message was created?
     */
    /*package*/ void showDate(boolean value) {
        myShowDate = value;
    }

    /**
     * Should the message include an abbreviated description of
     * the level at which the message was posted?
     */
    /*package*/ void showLevel(boolean value) {
        myShowLevel = value;
    }

    /**
     * Should the message include the method name, file name, and
     * line number?
     */
    /*package*/ void showLocation(boolean value) {
        myShowLocation = value;
    }

    /**
     * Should the message include the subsystem name?
     */
    /*package*/ void showSubsystem(boolean value) {
        myShowSubsystem = value;
    }

    /**
     * Should the message include the time at which the
     * message was created?
     */
    /*package*/ void showTime(boolean value) {
        myShowTime = value;
    }
    /** 
     * Convert the given message into a string, obeying 'show'
     * controls set earlier.
     */
    /*package*/ String toString(TraceMessage message) {
        buffer.setLength(0);
        buffer.append("=== ");
        if (myShowDate) {
            buffer.append(TraceDateToString.dateString(message.date));
            buffer.append(' ');
        } 
        if (myShowTime) { 
            buffer.append(TraceDateToString.timeString(message.date));
            buffer.append(' ');
        } 
        if (myShowLocation) {
            if (TraceController.debugTraceCaller) {
                System.err.println("****************************************");
                System.err.println(message.subsystem + ": " + message.message);
            }
            TraceCaller frameData = new TraceCaller(message.exception);
            buffer.append('(');
            buffer.append(frameData.methodName);
            buffer.append(':');
            buffer.append(frameData.fileName);
            buffer.append(':');
            buffer.append(frameData.lineNumber);
            buffer.append(") ");
        }

        if (myShowLevel) {
            buffer.append(TraceLevelTranslator.terse(message.level));
        }
        
        // Experimenting with multi-line output.
        if (myShowDate || myShowTime || myShowLocation) {
            buffer.append(lineSeparator);
        }

        // note: subsystem must be delimited because it might
        // contain blanks.
        if (myShowSubsystem) {
            buffer.append(message.subsystem);
            buffer.append(": ");
        }
        buffer.append(message.message);

        if (message.object != null) {
            if (message.object instanceof Throwable) {
                StringWriter sw = new StringWriter();
                PrintWriter stream = new PrintWriter(sw);
                ((Throwable)(message.object)).printStackTrace(stream);
                String stack = sw.getBuffer().toString();
                buffer.append(lineSeparator);
                buffer.append(stack);
            } else {
                buffer.append(" : ");
                buffer.append(message.object);
            }
        }
        return buffer.toString();
    }
}
